/**********************************************************
 ** gPND maker v.0.1 BETA 1                              **
 ** By: Steven Arnow (steven@slaeshjag.org)              **
 ** License: LGPL v.2                                    **
 **********************************************************/

#include "gpnd.h"


int gpndArgIsPath(char *str) {
	if (strstr(str, "/") != NULL)
		return 1;
	if (strstr(str, "..") != NULL)
		return 1;
	if (strstr(str, "~") != NULL)
		return 1;
	return 0;
}

	
int gpndLocaleExists(GPND_MAIN *m, const char *locale) {
	int i;

	for (i = 0; i < m->localelen; i++)
		if (strcmp(locale, m->locales[i]) == 0)
			return 1;
	return 0;
}		


int gpndPathIsRelative(char *str) {
	if (str[0] == '/')
		return 0;
	return 1;
}


int gpndFileInPNDExists(GPND_MAIN *m, char *file) {
	char *path;
	FILE *fp;
	
	if ((path = malloc(strlen(m->dirname) + strlen(file) + 2)) == NULL) {
		printf("Out of RAM!\n");
		return 0;
	}

	sprintf(path, "%s/%s", m->dirname, file);

	if ((fp = fopen(path, "r")) == NULL)
		return 0;
	fclose(fp);
	return 1;
}	


void gpndUpdateButtonSensitivity(GPND_MAIN *m) {
	int i, mkpnd;
	gtk_widget_set_sensitive(GTK_WIDGET(m->gtkstruct.addApp), 0);
	gtk_widget_set_sensitive(GTK_WIDGET(m->gtkstruct.renApp), 0);
	gtk_widget_set_sensitive(GTK_WIDGET(m->gtkstruct.delApp), 0);
	gtk_widget_set_sensitive(GTK_WIDGET(m->gtkstruct.appName), 0);
	gtk_widget_set_sensitive(GTK_WIDGET(m->gtkstruct.appinfo), 0);
	gtk_widget_set_sensitive(GTK_WIDGET(m->gtkstruct.apprun), 0);
	gtk_widget_set_sensitive(GTK_WIDGET(m->gtkstruct.cat), 0);
	gtk_widget_set_sensitive(GTK_WIDGET(m->gtkstruct.savepnd), 0);
	gtk_widget_set_sensitive(GTK_WIDGET(m->gtkstruct.appndesc), 0);
	gtk_widget_set_sensitive(GTK_WIDGET(m->gtkstruct.applist), 1);
	
	
	if (m->activeApp != -1) {
		gtk_widget_set_sensitive(GTK_WIDGET(m->gtkstruct.seldir), 0);
		if (m->flags & 0x10000 && !(m->flags & 0x1)) {
			gtk_widget_set_sensitive(GTK_WIDGET(m->gtkstruct.appinfo), 1);
			gtk_widget_set_sensitive(GTK_WIDGET(m->gtkstruct.addApp), 1);
			gtk_widget_set_sensitive(GTK_WIDGET(m->gtkstruct.renApp), 1);
			gtk_widget_set_sensitive(GTK_WIDGET(m->gtkstruct.delApp), 1);
			gtk_widget_set_sensitive(GTK_WIDGET(m->gtkstruct.appName), 1);
		}
		if (m->app[m->activeApp].flags & 0x100 && !(m->flags & 0x1)) {
			gtk_widget_set_sensitive(GTK_WIDGET(m->gtkstruct.apprun), 1);
			if (m->app[m->activeApp].flags & 0x200 && !(m->flags & 0x1)) {
				gtk_widget_set_sensitive(GTK_WIDGET(m->gtkstruct.appndesc), 1);
				if (m->app[m->activeApp].flags & 0x400 && !(m->flags & 0x1)) {
					gtk_widget_set_sensitive(GTK_WIDGET(m->gtkstruct.cat), 1);
					i = 0;
					mkpnd = 1;
					do {
						if (m->app[i].flags & GPND_APP_APPISVALID)
							if (!(m->app[i].flags & 0x800)) {
								mkpnd = 0;
							}
						i++;
					} while (m->app[i-1].flags & GPND_APP_THEREISNEXTAPP);
					
					if (mkpnd == 1 && !(m->flags & 0x1)) {
						gtk_widget_set_sensitive(GTK_WIDGET(m->gtkstruct.savepnd), 1);
					}
				}
			}
		}
	} else {
		if (m->flags & 0x10000 && !(m->flags & 0x1)) {
			gtk_widget_set_sensitive(GTK_WIDGET(m->gtkstruct.addApp), 1);
			gtk_widget_set_sensitive(GTK_WIDGET(m->gtkstruct.renApp), 1);
			gtk_widget_set_sensitive(GTK_WIDGET(m->gtkstruct.delApp), 1);
			gtk_widget_set_sensitive(GTK_WIDGET(m->gtkstruct.appName), 1);
		} else {
			gtk_widget_set_sensitive(GTK_WIDGET(m->gtkstruct.addApp), 0);
			gtk_widget_set_sensitive(GTK_WIDGET(m->gtkstruct.renApp), 0);
			gtk_widget_set_sensitive(GTK_WIDGET(m->gtkstruct.delApp), 0);
			gtk_widget_set_sensitive(GTK_WIDGET(m->gtkstruct.appName), 0);
			gtk_widget_set_sensitive(GTK_WIDGET(m->gtkstruct.appinfo), 0);
			gtk_widget_set_sensitive(GTK_WIDGET(m->gtkstruct.apprun), 0);
			gtk_widget_set_sensitive(GTK_WIDGET(m->gtkstruct.cat), 0);
			gtk_widget_set_sensitive(GTK_WIDGET(m->gtkstruct.savepnd), 0);
			gtk_widget_set_sensitive(GTK_WIDGET(m->gtkstruct.appndesc), 0);
		}

		if (m->noofApps) {
			gtk_widget_set_sensitive(GTK_WIDGET(m->gtkstruct.seldir), 0);
		} else {
			gtk_widget_set_sensitive(GTK_WIDGET(m->gtkstruct.seldir), 1);
		}
	}

	if (!(m->flags & 0x1) && m->noofApps) {
		gtk_widget_set_sensitive(GTK_WIDGET(m->gtkstruct.applist), 1);
	} else
		gtk_widget_set_sensitive(GTK_WIDGET(m->gtkstruct.applist), 0);
	
	return;
}


void gpndEscapeXMLData(char *str) {
	int i;

	for (i = 0; str[i] != 0; i++) {
		if (str[i] == '"')
			str[i] = '\'';
		else if (str[i] == '<')
			str[i] = '[';
		else if (str[i] == '>')
			str[i] = ']';
	}

	return;
}


int gpndGetLocaleIndex(GPND_MAIN *m, const char *ndesc) {
	int i = 0, index = -1;
	
	do {
		if (m->app[m->activeApp].ndesc[i].flags & GPND_APP_APPISVALID)
			if (strcmp(m->app[m->activeApp].ndesc[i].lang, ndesc) == 0)
				index = i;
		i++;
	} while (m->app[m->activeApp].ndesc[i-1].flags & GPND_APP_THEREISNEXTAPP);
	

	return index;
}


int gpndGetCatIndex(GPND_MAIN *m, int cat, const char *subcat) {
	int i = 0, index = -1;
	
	do {
		if (m->app[m->activeApp].cat[i].flags & GPND_APP_APPISVALID)
			if (strcmp(m->app[m->activeApp].cat[i].subcat, subcat) == 0 && m->app[m->activeApp].cat[i].cat == cat)
				index = i;
		i++;
	} while (m->app[m->activeApp].cat[i-1].flags & GPND_APP_THEREISNEXTAPP);
	

	return index;
}


int gpndGetAppIndex(GPND_MAIN *m, const char *app) {
	int i = 0, index = -1;
	
	do {
		if (m->app[i].flags & GPND_APP_APPISVALID)
			if (strcmp(m->app[i].appname, app) == 0)
				index = i;
		i++;
	} while (m->app[i-1].flags & GPND_APP_THEREISNEXTAPP);
	

	return index;
}


void gpndShowError(GPND_MAIN *m, char *message, int flags) {
	GtkWidget *dialog;
	GtkWindow *window;

	if (flags)
		window = (GtkWindow *) m->currentWindow;
	else 
		window = (GtkWindow *) m->gtkstruct.window;
	
	dialog = gtk_message_dialog_new(window,
		GTK_DIALOG_DESTROY_WITH_PARENT,
		GTK_MESSAGE_ERROR,
		GTK_BUTTONS_OK,
		(char *) message);
	gtk_window_set_title(GTK_WINDOW(dialog), "Fatal error");
	gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);
	
	return;
}



void gpndLoadLocales(GPND_MAIN *m) {
	FILE *fp;
	int i;
	char buff[20];

	m->locales = NULL;
	
	if ((fp = fopen("locales", "r")) == NULL) {
		gpndShowError(m, "Failed to open locales file! Make sure it is in the same directory as this application and try again.", 0);
		exit(254);
	}

	for (i = 0; !feof(fp); i++) {
		if ((m->locales = realloc(m->locales, sizeof(char *) * (i + 1))) == NULL) {
			gpndShowError(m, "Out of RAM!", 0);
			exit(2);
		}
		
		fgets(buff, 20, fp);
		m->locales[i] = malloc(strlen(buff));
		buff[strlen(buff) - 1] = '\0';

		sprintf(m->locales[i], "%s", buff);
	}

	m->localelen = i;

	return;
}


void gpndAddToListView(GPND_MAIN *m, GtkWidget *list, const gchar *str) {
	GtkListStore *store;
	GtkTreeIter iter;

	store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(list)));

	gtk_list_store_append(store, &iter);
	gtk_list_store_set(store, &iter, 0, str, -1);

	return;
}


void gpndInitApp(GPND_MAIN *m) {
	m->app = malloc(sizeof(GPND_APP));
	m->app->flags = 0;

	return;
}


void gpndInitCat(GPND_MAIN *m, int i) {
	m->app[i].cat = malloc(sizeof(GPND_CATEGORY_STRUCT));
	m->app[i].cat->flags = 0;
	m->app[i].cat->cat = 0;
	m->app[i].cat->subcat = NULL;
	m->app[i].noofCat = 0;

	return;
}


void gpndInitNDesc(GPND_MAIN *m, int i) {
	m->app[i].ndesc = malloc(sizeof(GPND_NDESC));
	m->app[i].ndesc->flags = 0;
	m->app[i].ndesc->name = NULL;

	return;
}


void gpndNDescDel(GtkWidget *widget, gpointer data) {
	GPND_MAIN *m = (GPND_MAIN *) data;
	char *list;
	int active;
	GtkTreeSelection *selection;
	GtkListStore *store;
	GtkTreeModel *model;
	GtkTreeIter  iter;
	GtkTextBuffer *buffer;

	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW (m->gtkndesc.desce));
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(m->gtkndesc.list));
	store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW (m->gtkndesc.list)));
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(m->gtkndesc.list));
	
	if (gtk_tree_model_get_iter_first(model, &iter) == FALSE) 
		return;
	
	gtk_tree_selection_get_selected(GTK_TREE_SELECTION(selection), &model, &iter);
	gtk_tree_model_get(model, &iter, 0, &list,  -1);
	active = gpndGetLocaleIndex(m, list);

	g_free(m->app[m->activeApp].ndesc[active].desc);
	m->app[m->activeApp].ndesc[active].desc = NULL;

	if (strcmp(m->app[m->activeApp].ndesc[active].lang, "en_US") == 0)
		m->app[m->activeApp].flags &= 0xFFFFFBFF;

	free(m->app[m->activeApp].ndesc[active].name);
	free(m->app[m->activeApp].ndesc[active].lang);
	m->app[m->activeApp].ndesc[active].name = NULL;
	m->app[m->activeApp].ndesc[active].lang = NULL;

	gtk_list_store_set(store, &iter, 0, m->app[m->activeApp].ndesc[active].lang, -1);
	
	if (gtk_tree_selection_get_selected(GTK_TREE_SELECTION(selection), &model, &iter)) {
		gtk_entry_set_text((GtkEntry *) m->gtkndesc.lange, "");
		gtk_entry_set_text((GtkEntry *) m->gtkndesc.namee, "");
		gtk_text_buffer_set_text(buffer, "", -1);
		
		gtk_list_store_remove(store, &iter);
		m->app[m->activeApp].ndesc[active].flags ^= GPND_APP_APPISVALID;
	}

	return;
}


void gpndNDescModifyEntry(GtkWidget *widget, gpointer data) {
	GPND_MAIN *m = (GPND_MAIN *) data;
	int active;
	char *desc, *list;
	GtkTextIter start, end;
	GtkTreeSelection *selection;
	GtkListStore *store;
	GtkTreeModel *model;
	GtkTreeIter  iter;
	GtkTextBuffer *buffer;

	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW (m->gtkndesc.desce));
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(m->gtkndesc.list));
	store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW (m->gtkndesc.list)));
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(m->gtkndesc.list));
	
	if (!gpndLocaleExists(m, gtk_entry_get_text((GtkEntry *) m->gtkndesc.lange))) {
		gpndShowError(m, "The entered locale does not exist!", GPND_MESSAGE_CHILD_WINDOW);
		return;
	}

	if (gpndGetLocaleIndex(m, gtk_entry_get_text((GtkEntry *) m->gtkndesc.lange)) != -1) {
		gpndShowError(m, "It does already exist a name/description entry for that locale!", GPND_MESSAGE_CHILD_WINDOW);
		return;
	}
	
	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW (m->gtkndesc.desce));
	
	gtk_text_buffer_get_start_iter(buffer, &start);
	gtk_text_buffer_get_end_iter(buffer, &end);


	if (strlen(gtk_entry_get_text((GtkEntry *) m->gtkndesc.namee)) == 0) {
		gpndShowError(m, "You must enter a name for the app!", GPND_MESSAGE_CHILD_WINDOW);
		return;
	}

	desc = gtk_text_buffer_get_text(buffer, &start, &end, FALSE);
	
	if (strlen(desc) == 0) {
		gpndShowError(m, "You must enter a description!", GPND_MESSAGE_CHILD_WINDOW);
		g_free(desc);
		return;
	}

	gtk_tree_selection_get_selected(GTK_TREE_SELECTION(selection), &model, &iter);
	gtk_tree_model_get(model, &iter, 0, &list,  -1);
	active = gpndGetLocaleIndex(m, list);

	g_free(m->app[m->activeApp].ndesc[active].desc);
	m->app[m->activeApp].ndesc[active].desc = desc;

	m->app[m->activeApp].ndesc[active].name = realloc(m->app[m->activeApp].ndesc[active].name, strlen(gtk_entry_get_text((GtkEntry *) m->gtkndesc.namee)) + 1);
	if (m->app[m->activeApp].ndesc[active].name != NULL)
		sprintf(m->app[m->activeApp].ndesc[active].name, "%s", gtk_entry_get_text((GtkEntry *) m->gtkndesc.namee));


	m->app[m->activeApp].ndesc[active].lang = realloc(m->app[m->activeApp].ndesc[active].lang, strlen(gtk_entry_get_text((GtkEntry *) m->gtkndesc.lange)) + 1);
	if (m->app[m->activeApp].ndesc[active].lang != NULL)
		sprintf(m->app[m->activeApp].ndesc[active].lang, "%s", gtk_entry_get_text((GtkEntry *) m->gtkndesc.lange));


	gtk_list_store_set(store, &iter, 0, m->app[m->activeApp].ndesc[active].lang, -1);

	return;
}


void gpndOnNDescListUpdate(GtkWidget *widget, gpointer data) {
	GPND_MAIN *m = (GPND_MAIN *) data;
	GtkTreeSelection *selection;
	GtkListStore *store;
	GtkTreeModel *model;
	GtkTreeIter  iter;
	GtkTextBuffer *buffer;
	int active;
	char *list;
	
	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW (m->gtkndesc.desce));
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(m->gtkndesc.list));
	store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW (m->gtkndesc.list)));
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(m->gtkndesc.list));

	
	
	gtk_tree_selection_get_selected(GTK_TREE_SELECTION(selection), &model, &iter);
	
	if (gtk_tree_selection_get_selected(GTK_TREE_SELECTION(selection), &model, &iter)) {
		gtk_tree_model_get(model, &iter, 0, &list,  -1);
		active = gpndGetLocaleIndex(m, list);

		if (active > -1) {
			gtk_entry_set_text((GtkEntry *) m->gtkndesc.lange, m->app[m->activeApp].ndesc[active].lang);
			gtk_entry_set_text((GtkEntry *) m->gtkndesc.namee, m->app[m->activeApp].ndesc[active].name);
			gtk_text_buffer_set_text(buffer, m->app[m->activeApp].ndesc[active].desc, -1);
		} else {
			gtk_entry_set_text((GtkEntry *) m->gtkndesc.lange, "");
			gtk_entry_set_text((GtkEntry *) m->gtkndesc.namee, "");
			gtk_text_buffer_set_text(buffer, "", -1);
		}
		
	}

	return;
}


void gpndCatDel(GtkWidget *widget, gpointer data) {
	GPND_MAIN *m = (GPND_MAIN *) data;
	char *list, *subcat;
	int active, cat;
	GtkTreeSelection *selection;
	GtkListStore *store;
	GtkTreeModel *model;
	GtkTreeIter  iter;
	

	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(m->gtkcat.list));
	store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW (m->gtkcat.list)));
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(m->gtkcat.list));

	if (gtk_tree_model_get_iter_first(model, &iter) == FALSE) 
		return;
	
	gtk_tree_selection_get_selected(GTK_TREE_SELECTION(selection), &model, &iter);
	gtk_tree_model_get(model, &iter, 0, &list,  -1);
	
	subcat = malloc(strlen(list));
	
	sscanf(list, "%i/%s", &cat, subcat);
	active = gpndGetCatIndex(m, cat, subcat);
	free(subcat);

	free(m->app[m->activeApp].cat[active].subcat);
	m->app[m->activeApp].cat[active].subcat = NULL;

	m->app[m->activeApp].noofCat--;
	
	if (m->app[m->activeApp].noofCat == 0)
		m->app[m->activeApp].flags ^= 0x800;
	
	if (gtk_tree_selection_get_selected(GTK_TREE_SELECTION(selection), &model, &iter)) {
		gtk_entry_set_text((GtkEntry *) m->gtkcat.sube, "");
		gtk_combo_box_set_active((GtkComboBox *) m->gtkcat.cate, 0);
		
		gtk_list_store_remove(store, &iter);
		m->app[m->activeApp].cat[active].flags ^= GPND_APP_APPISVALID;
	}

	return;
}


void gpndCatModifyEntry(GtkWidget *widget, gpointer data) {
	GPND_MAIN *m = (GPND_MAIN *) data;
	int active, cat;
	char *list, *subcat;
	GtkTreeSelection *selection;
	GtkListStore *store;
	GtkTreeModel *model;
	GtkTreeIter  iter;

	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(m->gtkcat.list));
	store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW (m->gtkcat.list)));
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(m->gtkcat.list));
	
	if (gpndGetCatIndex(m, gtk_combo_box_get_active((GtkComboBox *) m->gtkcat.cate), gtk_entry_get_text((GtkEntry *) m->gtkcat.sube)) != -1) {
		gpndShowError(m, "It does already exist a entry with that category/subcategory combination!", GPND_MESSAGE_CHILD_WINDOW);
		return;
	}

	gtk_tree_selection_get_selected(GTK_TREE_SELECTION(selection), &model, &iter);
	gtk_tree_model_get(model, &iter, 0, &list,  -1);

	subcat = malloc(strlen(list));
	sscanf(list, "%i/%s", &cat, subcat);
	active = gpndGetCatIndex(m, cat, subcat);
	free(subcat);

	m->app[m->activeApp].cat[active].subcat = realloc(m->app[m->activeApp].cat[active].subcat, strlen(gtk_entry_get_text((GtkEntry *) m->gtkcat.sube)) + 1);
	if (m->app[m->activeApp].cat[active].subcat != NULL)
		sprintf(m->app[m->activeApp].cat[active].subcat, "%s", gtk_entry_get_text((GtkEntry *) m->gtkcat.sube));

	m->app[m->activeApp].cat[active].cat = gtk_combo_box_get_active((GtkComboBox *) m->gtkcat.cate);
	g_free(list);

	subcat = malloc(strlen(gtk_entry_get_text((GtkEntry *) m->gtkcat.sube)) + 4);
	sprintf(subcat, "%i/%s", gtk_combo_box_get_active((GtkComboBox *) m->gtkcat.cate), gtk_entry_get_text((GtkEntry *) m->gtkcat.sube));
	
	gtk_list_store_set(store, &iter, 0, subcat, -1);
	free(subcat);

	return;
}


void gpndAddCat(GtkWidget *widget, gpointer data) {
	int i;
	GPND_MAIN *m = (GPND_MAIN *) data;
	char *list;
	

	if (gpndGetCatIndex(m, gtk_combo_box_get_active((GtkComboBox *) m->gtkcat.cate), gtk_entry_get_text((GtkEntry *) m->gtkcat.sube)) != -1) {
		gpndShowError(m, "It does already exist a entry with that category/subcategory combination!", GPND_MESSAGE_CHILD_WINDOW);
		return;
	}
	
	for (i = 0; m->app[m->activeApp].cat[i].flags & GPND_APP_THEREISNEXTAPP && m->app[m->activeApp].cat[i].flags & GPND_APP_APPISVALID; i++);

	if (m->app[m->activeApp].cat[i].flags & GPND_APP_APPISVALID) {
		m->app[m->activeApp].cat[i].flags |= GPND_APP_THEREISNEXTAPP;
		i++;
		if ((m->app[m->activeApp].cat = realloc(m->app[m->activeApp].cat, sizeof(GPND_CATEGORY_STRUCT) * (i + 1))) == NULL) {
			gpndShowError(m, "Out of RAM!", 0);
			exit(255);
		}
		m->app[m->activeApp].cat[i].flags = GPND_APP_APPISVALID;
	} else
		m->app[m->activeApp].cat[i].flags |= GPND_APP_APPISVALID;

	m->app[m->activeApp].cat[i].subcat = malloc(strlen(gtk_entry_get_text((GtkEntry *) m->gtkcat.sube)) + 1);
	if (m->app[m->activeApp].cat[i].subcat != NULL)
		sprintf(m->app[m->activeApp].cat[i].subcat, "%s", gtk_entry_get_text((GtkEntry *) m->gtkcat.sube));


	list = malloc(strlen(gtk_entry_get_text((GtkEntry *) m->gtkcat.sube)) + 4);
	sprintf(list, "%i/%s", gtk_combo_box_get_active((GtkComboBox *) m->gtkcat.cate), gtk_entry_get_text((GtkEntry *) m->gtkcat.sube));
	m->app[m->activeApp].cat[i].cat = gtk_combo_box_get_active((GtkComboBox *) m->gtkcat.cate);
	m->app[m->activeApp].noofCat++;
	m->app[m->activeApp].flags |= 0x800;
	gpndUpdateButtonSensitivity(m);
	gpndAddToListView(m, m->gtkcat.list, list);
	free(list);
	
	
	return;
}


void gpndOnCatListUpdate(GtkWidget *widget, gpointer data) {
	GPND_MAIN *m = (GPND_MAIN *) data;
	GtkTreeSelection *selection;
	GtkListStore *store;
	GtkTreeModel *model;
	GtkTreeIter  iter;
	int active, cat;
	char *list, *subcat;
	
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(m->gtkcat.list));
	store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW (m->gtkcat.list)));
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(m->gtkcat.list));

	
	
	gtk_tree_selection_get_selected(GTK_TREE_SELECTION(selection), &model, &iter);
	
	if (gtk_tree_selection_get_selected(GTK_TREE_SELECTION(selection), &model, &iter)) {
		gtk_tree_model_get(model, &iter, 0, &list,  -1);
		subcat = malloc(strlen(list));
		sscanf(list, "%i/%s", &cat, subcat);
		active = gpndGetCatIndex(m, cat, subcat);

		
		if (active > -1) {
			gtk_entry_set_text((GtkEntry *) m->gtkcat.sube, m->app[m->activeApp].cat[active].subcat);
			gtk_combo_box_set_active((GtkComboBox *) m->gtkcat.cate, m->app[m->activeApp].cat[active].cat);
		} else {
			gtk_entry_set_text((GtkEntry *) m->gtkcat.sube, "");
			gtk_combo_box_set_active((GtkComboBox *) m->gtkcat.cate, 0);
		}
		
	}

	return;
}


void gpndAddNDesc(GtkWidget *widget, gpointer data) {
	gchar *desc;
	int i;
	GPND_MAIN *m = (GPND_MAIN *) data;
	GtkTextIter start, end;
	GtkTextBuffer *buffer;
	

	if (gpndGetLocaleIndex(m, gtk_entry_get_text((GtkEntry *) m->gtkndesc.lange)) != -1) {
		gpndShowError(m, "It does already exist a name/description entry for that locale!", GPND_MESSAGE_CHILD_WINDOW);
		return;
	}

	if (!gpndLocaleExists(m, gtk_entry_get_text((GtkEntry *) m->gtkndesc.lange))) {
		gpndShowError(m, "The entered locale does not exist!", GPND_MESSAGE_CHILD_WINDOW);
		return;
	}
	
	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW (m->gtkndesc.desce));
	
	gtk_text_buffer_get_start_iter(buffer, &start);
	gtk_text_buffer_get_end_iter(buffer, &end);


	if (strlen(gtk_entry_get_text((GtkEntry *) m->gtkndesc.namee)) == 0) {
		gpndShowError(m, "You must enter a name for the app!", GPND_MESSAGE_CHILD_WINDOW);
		return;
	}

	desc = gtk_text_buffer_get_text(buffer, &start, &end, FALSE);
	
	if (strlen(desc) == 0) {
		gpndShowError(m, "You must enter a description!", GPND_MESSAGE_CHILD_WINDOW);
		g_free(desc);
		return;
	}
	
	

	for (i = 0; m->app[m->activeApp].ndesc[i].flags & GPND_APP_THEREISNEXTAPP && m->app[m->activeApp].ndesc[i].flags & GPND_APP_APPISVALID; i++);

	
	if (m->app[m->activeApp].ndesc[i].flags & GPND_APP_APPISVALID) {
		m->app[m->activeApp].ndesc[i].flags |= GPND_APP_THEREISNEXTAPP;
		i++;
		if ((m->app[m->activeApp].ndesc = realloc(m->app[m->activeApp].ndesc, sizeof(GPND_NDESC) * (i + 1))) == NULL) {
			gpndShowError(m, "Out of RAM!", 0);
			exit(255);
		}
		m->app[m->activeApp].ndesc[i].flags = GPND_APP_APPISVALID;
	} else
		m->app[m->activeApp].ndesc[i].flags |= GPND_APP_APPISVALID;

	m->app[m->activeApp].ndesc[i].name = malloc(strlen(gtk_entry_get_text((GtkEntry *) m->gtkndesc.namee)) + 1);
	if (m->app[m->activeApp].ndesc[i].name != NULL)
		sprintf(m->app[m->activeApp].ndesc[i].name, "%s", gtk_entry_get_text((GtkEntry *) m->gtkndesc.namee));


	m->app[m->activeApp].ndesc[i].lang = malloc(strlen(gtk_entry_get_text((GtkEntry *) m->gtkndesc.lange)) + 1);
	if (m->app[m->activeApp].ndesc[i].lang != NULL)
		sprintf(m->app[m->activeApp].ndesc[i].lang, "%s", gtk_entry_get_text((GtkEntry *) m->gtkndesc.lange));
	
	m->app[m->activeApp].ndesc[i].desc = desc;
	
	gpndUpdateButtonSensitivity(m);
	gpndAddToListView(m, m->gtkndesc.list, gtk_entry_get_text((GtkEntry *) m->gtkndesc.lange));
	
	
	return;
}


void gpndAddApp(GtkWidget *widget, gpointer data) {
	const gchar *appname;
	int i;
	GPND_MAIN *m = (GPND_MAIN *) data;

	appname = gtk_entry_get_text((GtkEntry *) m->gtkstruct.appName);

	if (gpndGetAppIndex(m, (char *) appname) != -1) {
		printf("index: %i\n", gpndGetAppIndex(m, appname));
		gpndShowError(m, "It does alredy exist a app with that name!", 0);
		return;
	}
	
	if (strlen(appname) > 0)
		gpndAddToListView(m, m->gtkstruct.applist, appname);
	else {
		gpndShowError(m, "You must enter a name for the app!", 0);
		return;
	}
	

	for (i = 0; m->app[i].flags & GPND_APP_THEREISNEXTAPP && m->app[i].flags & GPND_APP_APPISVALID; i++);
	
	if (m->app[i].flags & GPND_APP_APPISVALID) {
		m->app[i].flags |= GPND_APP_THEREISNEXTAPP;
		i++;
		if ((m->app = realloc(m->app, sizeof(GPND_APP) * (i + 1))) == NULL) {
			gpndShowError(m, "Out of RAM!", 0);
			exit(255);
		}

		m->app[i].flags = GPND_APP_APPISVALID;
	} else
		m->app[i].flags |= GPND_APP_APPISVALID;
	
	
	m->app[i].author = NULL;
	m->app[i].mail = NULL;
	m->app[i].http = NULL;
	m->app[i].desc = NULL;
	m->app[i].icon = NULL;
	m->app[i].version = NULL;
	m->app[i].uid = NULL;
	m->app[i].appdata = NULL;
	m->app[i].appname = NULL;
	m->app[i].startdir = NULL;
	m->app[i].execfile = NULL;
	m->app[i].args = NULL;
	m->app[i].cat = 0;
	m->app[i].cpufreq = 0;
	m->app[i].allowMultitasking = 1;
	m->app[i].standalone = 0;
	m->app[i].x11 = 2;

	m->app[i].appname = realloc(m->app[i].appname, strlen(appname) + 1);
	if (m->app[i].appname != NULL)
		sprintf(m->app[i].appname, "%s", appname);
		
	m->app[i].ndesc = malloc(sizeof(GPND_NAMEDESC_STRUCT));
	m->app[i].ndesc->flags = 0;
	gpndInitCat(m, i);

	
	m->noofApps++;
	gpndUpdateButtonSensitivity(m);
	gtk_entry_set_text((GtkEntry *) m->gtkstruct.appName, "");
	
	return;
}


void gpndFreeApp(GPND_MAIN *m, int i) {
	free(m->app[i].author);
	free(m->app[i].mail);
	free(m->app[i].http);
	free(m->app[i].desc);
	free(m->app[i].icon);
	free(m->app[i].version);
	free(m->app[i].uid);
	free(m->app[i].appdata);
	free(m->app[i].appname);
	free(m->app[i].startdir);
	free(m->app[i].execfile);
	free(m->app[i].args);

	return;
}


void gpndDelApp(GtkWidget *widget, gpointer data) {
	GPND_MAIN *m = (GPND_MAIN *) data;
	GtkTreeSelection *selection;
	GtkListStore *store;
	GtkTreeModel *model;
	GtkTreeIter  iter;

	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(m->gtkstruct.applist));
	store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW (m->gtkstruct.applist)));
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(m->gtkstruct.applist));

	if (gtk_tree_model_get_iter_first(model, &iter) == FALSE) 
		return;
	
	if (gtk_tree_selection_get_selected(GTK_TREE_SELECTION(selection), &model, &iter)) {
		gtk_entry_set_text((GtkEntry *) m->gtkstruct.appName, "");
		gtk_list_store_remove(store, &iter);
		m->app[m->activeApp].flags ^= GPND_APP_APPISVALID;
		gpndFreeApp(m, m->activeApp);
		m->activeApp = -1;
		m->noofApps--;
		gpndUpdateButtonSensitivity(m);
	}

	return;
}


void gpndRenApp(GtkWidget *widget, gpointer data) {
	GPND_MAIN *m = (GPND_MAIN *) data;
	GtkTreeSelection *selection;
	GtkListStore *store;
	GtkTreeModel *model;
	GtkTreeIter  iter;
	const gchar *app, *appname;

	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(m->gtkstruct.applist));
	store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW (m->gtkstruct.applist)));
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(m->gtkstruct.applist));

	appname = gtk_entry_get_text((GtkEntry *) m->gtkstruct.appName);

	if (gpndGetAppIndex(m, appname) != -1) {
		gpndShowError(m, "It does alredy exist a app with that name!", 0);
		return;
	}
	
	if (strlen(appname) > 0);
	else {
		gpndShowError(m, "You must enter a name for the app!", 0);
		return;
	}

	if (gtk_tree_model_get_iter_first(model, &iter) == FALSE) 
		return;
	
	if (gtk_tree_selection_get_selected(GTK_TREE_SELECTION(selection), &model, &iter)) {
		app = gtk_entry_get_text((GtkEntry *) m->gtkstruct.appName);
		m->app[m->activeApp].appname = (char *) app;
		gtk_list_store_set(store, &iter, 0, app, -1);
	}

	return;
}


void gpndOnAppListUpdate(GtkWidget *widget, gpointer data) {
	GPND_MAIN *m = (GPND_MAIN *) data;
	GtkTreeSelection *selection;
	GtkListStore *store;
	GtkTreeModel *model;
	GtkTreeIter  iter;
	char *app;

	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(m->gtkstruct.applist));
	store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW (m->gtkstruct.applist)));
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(m->gtkstruct.applist));

	if (gtk_tree_model_get_iter_first(model, &iter) == FALSE) 
		return;
	
	if (gtk_tree_selection_get_selected(GTK_TREE_SELECTION(selection), &model, &iter)) {
		gtk_tree_model_get(model, &iter, 0, &app,  -1);
		gtk_entry_set_text((GtkEntry *) m->gtkstruct.appName, app);
		m->activeApp = gpndGetAppIndex(m, app);
		gpndUpdateButtonSensitivity(m);
	}

	return;
}


void gpndInitListView(GPND_MAIN *m, GtkWidget *list) {
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	GtkListStore *store;

	renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes("List Items", renderer, "text", 0, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column);

	store = gtk_list_store_new(1, G_TYPE_STRING);

	gtk_tree_view_set_model(GTK_TREE_VIEW(list), GTK_TREE_MODEL(store));

	g_object_unref(store);

	return;
}


void gpndGetDir(GtkWidget *widget, gpointer data) {
	GPND_MAIN *m = (GPND_MAIN *) data;
	GtkWidget *dialog;
	
	dialog = gtk_file_chooser_dialog_new("Select directory", (gpointer) m->gtkstruct.window, GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
	
	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
		if (m->dirname == NULL)
			g_free(m->dirname);
		
		m->dirname = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
		printf("Got: %s\n", m->dirname);
	}

	m->flags |= 0x10000;
	gpndUpdateButtonSensitivity(m);
	gtk_widget_destroy(dialog);

	return;
}


void gpndGetTargetPND(GtkWidget *widget, gpointer data) {
	GPND_MAIN *m = (GPND_MAIN *) data;
	GtkWidget *dialog;
	
	dialog = gtk_file_chooser_dialog_new("Select target PND", (gpointer) m->gtkstruct.window, GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
	gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(dialog), TRUE);
	gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(dialog), "untitled.pnd");
	
	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
		if (m->dirname == NULL)
			g_free(m->target);
		
		m->target = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
		gtk_entry_set_text((GtkEntry *) m->gtksavepnd.savetoe, m->target);
	}
	
	gtk_widget_destroy(dialog);
	
	return;
}


const char *gpndGetCat(GPND_MAIN *m, int i) {
	printf("Request about category #%i\n", m->app[m->activeApp].cat[i].cat);
	switch (m->app[m->activeApp].cat[i].cat) {
		case 0:
			return "AudioVideo";
			break;
		case 1:
			return "Audio";
			break;
		case 2:
			return "Video";
			break;
		case 3:
			return "Development";
			break;
		case 4:	
			return "Education";
			break;
		case 5:
			return "Game";
			break;
		case 6:
			return "Graphics";
			break;
		case 7:
			return "Network";
			break;
		case 8:
			return "Office";
			break;
		case 9:
			return "Settings";
			break;
		case 10:
			return "System";
			break;
		case 11:
			return "Utility";
			break;
	}

	return NULL;
}


char *gpndIntToBoolChar(int i) {
	char *ret;

	ret = malloc(6);
	
	if (!i)
		sprintf(ret, "false");
	else
		sprintf(ret, "true");

	return ret;
}


int gpndMakePXML(GPND_MAIN *m) {
	char *pxml, *x11, *standalone, *background;
	int freq, i, j, major, minor, release, build;
	FILE *pxmlfp;
	
	pxml = malloc(strlen(m->dirname) + 10);
	sprintf(pxml, "%s/PXML.xml", m->dirname);

	if ((pxmlfp = fopen(pxml, "w")) == NULL) {
		printf("Could not create PXML.xml!\n");
		return 1;
	}
printf("Herro!\n");
	fprintf(pxmlfp, "<!-- Generator: gPND Beta 1 -->\n\n");
	fprintf(pxmlfp, "<PXML xmlns=\"http://openpandora.org/namespaces/PXML\">\n");

	i = 0;
	
	do {
		if (m->app[i].flags & GPND_APP_APPISVALID) {
			gpndEscapeXMLData(m->app[i].uid);
			fprintf(pxmlfp, "\t<application id=\"%s\"", m->app[i].uid);
			
			if (m->app[i].appdata != NULL && strlen(m->app[i].appdata)) {
				gpndEscapeXMLData(m->app[i].appdata);
				fprintf(pxmlfp, " appdata=\"%s\"", m->app[i].appdata);
			}
			
			fprintf(pxmlfp, ">\n");

			j = 0;
			do {
				if (m->app[i].ndesc[j].flags & GPND_APP_APPISVALID) {
					gpndEscapeXMLData(m->app[i].ndesc[j].lang);
					gpndEscapeXMLData(m->app[i].ndesc[j].name);
					fprintf(pxmlfp, "\t\t<title lang=\"%s\">%s</title>\n", m->app[i].ndesc[j].lang, m->app[i].ndesc[j].name);
					gpndEscapeXMLData(m->app[i].ndesc[j].lang);
					gpndEscapeXMLData(m->app[i].ndesc[j].desc);
					fprintf(pxmlfp, "\t\t<description lang=\"%s\">%s</description>\n", m->app[i].ndesc[j].lang, m->app[i].ndesc[j].desc);
				}
				j++;
			} while (m->app[i].ndesc[j-1].flags & GPND_APP_THEREISNEXTAPP);


			x11 = malloc(7);
		
			if (m->app[i].x11 == 0)
				sprintf(x11, "req");
			else if (m->app[i].x11 == 1)
				sprintf(x11, "stop");
			else
				sprintf(x11, "ignore");
		
			standalone = gpndIntToBoolChar(!m->app[i].standalone);
			background = gpndIntToBoolChar(m->app[i].allowMultitasking);
			
			fprintf(pxmlfp, "\t\t<exec background=\"%s\"", background);
			if (m->app[i].startdir != NULL && strlen(m->app[i].startdir) > 0) {
				gpndEscapeXMLData(m->app[i].startdir);
				fprintf(pxmlfp, " startdir=\"%s\"", m->app[i].startdir);
			} else
				fprintf(pxmlfp, " startdir=\"./\"");

			if (m->app[i].args != NULL && strlen(m->app[i].args) > 0) {
				gpndEscapeXMLData(m->app[i].args);
				fprintf(pxmlfp, " arguments=\"%s\"", m->app[i].args);
			}

			gpndEscapeXMLData(m->app[i].execfile);
			fprintf(pxmlfp, " standalone=\"%s\" command=\"%s\" x11=\"%s\" />\n", standalone, m->app[i].execfile, x11);

			free(x11);
			free(standalone);
			free(background);

			
			if (m->app[i].icon != NULL && strlen(m->app[i].icon) > 0) {
				gpndEscapeXMLData(m->app[i].icon);
				fprintf(pxmlfp, "\t\t<icon src=\"%s\" />\n", m->app[i].icon);
			}

			gpndEscapeXMLData(m->app[i].author);
			fprintf(pxmlfp, "\t\t<author name=\"%s\"", m->app[i].author);

			if (m->app[i].http != NULL && strlen(m->app[i].http) > 0) {
				gpndEscapeXMLData(m->app[i].http);
				fprintf(pxmlfp, " website=\"%s\"", m->app[i].http);
			}

			if (m->app[i].mail != NULL && strlen(m->app[i].mail) > 0) {
				gpndEscapeXMLData(m->app[i].mail);
				fprintf(pxmlfp, " email=\"%s\"", m->app[i].mail);
			}

			fprintf(pxmlfp, " />\n");

			sscanf(m->app[i].version, "%i,%i,%i,%i", &major, &minor, &release, &build);
			fprintf(pxmlfp, "\t\t<version major=\"%i\" minor=\"%i\" release=\"%i\" build=\"%i\" />\n", major, minor, release, build);

			fprintf(pxmlfp, "\n\t\t\n\t\t<categories>\n");
			
			j = 0;
			do {
				if (m->app[i].cat[j].flags & GPND_APP_APPISVALID) {
					fprintf(pxmlfp, "\t\t\t<category name=\"%s\">\n", gpndGetCat(m, j));
					if (m->app[i].cat[j].subcat != NULL && strlen(m->app[i].cat[j].subcat) > 0) {
						gpndEscapeXMLData(m->app[i].cat[j].subcat);
						fprintf(pxmlfp, "\t\t\t\t<subcategory name=\"%s\" />\n", m->app[i].cat[j].subcat);
					}

					fprintf(pxmlfp, "\t\t\t</category>\n");
				}
				j++;
			}  while (m->app[i].cat[j-1].flags & GPND_APP_THEREISNEXTAPP);

			fprintf(pxmlfp, "\t\t</categories>\n\t\t\n");

			if (m->app[i].cpufreq != 0) {
				freq = (m->app[i].cpufreq + 1) * 100;
				fprintf(pxmlfp, "\t\t<clockspeed frequency=\"%i\" />\n", freq);
			}
			
			fprintf(pxmlfp, "\t</application>\n\t\n");
		}
		
		i++;
	} while (m->app[i-1].flags & GPND_APP_THEREISNEXTAPP);

	fprintf(pxmlfp, "</PXML>\n");

	fclose(pxmlfp);

	return 0;
}

	

int gpndMakePND(GtkWidget *widget, gpointer data) {
	GPND_MAIN *m = (GPND_MAIN *) data;
	char *cmd, *fs;

	cmd = NULL;
	
	if (gpndMakePXML(m)) {
		gpndShowError(m, "Failed to create PXML.xml!\nMake sure the source media is writable.", 0);
		return 0;
	}
	fs = gtk_combo_box_get_active_text((GtkComboBox *) m->gtksavepnd.fse);

	if (strcmp(fs, "ISO9660") == 0) {
		cmd = malloc(24 + strlen(m->dirname) + strlen(gtk_entry_get_text((GtkEntry *) m->gtksavepnd.savetoe)) + 1);
		sprintf(cmd, "genisoimage -no-pad -o %s %s", gtk_entry_get_text((GtkEntry *) m->gtksavepnd.savetoe), m->dirname);
	} else if (strcmp(fs, "SquashFS") == 0) {
		cmd = malloc(32 + strlen(m->dirname) + strlen(gtk_entry_get_text((GtkEntry *) m->gtksavepnd.savetoe)) + 1);
		sprintf(cmd, "mksquashfs %s %s -noappend -noappend", m->dirname, gtk_entry_get_text((GtkEntry *) m->gtksavepnd.savetoe));
	} else if (strcmp(fs, "CramFS") == 0) {
		cmd = malloc(10 + strlen(m->dirname) + strlen(gtk_entry_get_text((GtkEntry *) m->gtksavepnd.savetoe)) + 1);
		sprintf(cmd, "mkcramfs %s %s", m->dirname, gtk_entry_get_text((GtkEntry *) m->gtksavepnd.savetoe));
	}
	
	if (system(cmd) != 0) {
		gpndShowError(m, "Failed to create the PND!\nMake sure the target media is writable.", 0);
		return 0;
	}

	cmd = realloc(cmd, 22 + strlen(m->dirname) + strlen(gtk_entry_get_text((GtkEntry *) m->gtksavepnd.savetoe)) + 1);
	sprintf(cmd, "cat \"%s/PXML.xml\" >> \"%s\"", m->dirname, gtk_entry_get_text((GtkEntry *) m->gtksavepnd.savetoe));

	if (system(cmd) != 0) {
		gpndShowError(m, "Failed to append the PND with PXML.xml!", 0);
		return 0;
	}
	
	free(cmd);
	g_free(fs);

	return 0;
}


int gpndInitStructs(GPND_MAIN *m) {
	m->flags = 0;
	m->dirname = NULL;

	m->activeApp = -1;

	m->noofApps = 0;

	return 0;
}


void gpndWindowKill(GtkWidget *widget, gpointer data) {
	GPND_MAIN *m = (GPND_MAIN *) data;
	
	g_signal_handler_disconnect(m->currentWindow, m->childKillID);
	
	gtk_widget_destroy(m->currentWindow);
	m->flags ^= 0x1;
	gpndUpdateButtonSensitivity(m);

	return;
}


void gpndGetNDesc(GtkWidget *widget, gpointer data) {
	GPND_MAIN *m = (GPND_MAIN *) data;

	if (gpndGetLocaleIndex(m, "en_US") == -1) {
		gpndShowError(m, "You must have a name/description for the locale »en_US«!", GPND_MESSAGE_CHILD_WINDOW);
		return;
	}

	m->app[m->activeApp].flags |= 0x400;
	gpndUpdateButtonSensitivity(m);
	gpndWindowKill(NULL, m);

	return;
}


void gpndGetAppGeneralData(GtkWidget *widget, gpointer data) {
	GPND_MAIN *m;
	m = (GPND_MAIN *) data;
	char *temp;

	if (strlen(gtk_entry_get_text((GtkEntry *) m->gtkgeneral.authore)) == 0) {
		gpndShowError(m, "You must specify the author!", GPND_MESSAGE_CHILD_WINDOW);
		return;
	}

	if (strlen(gtk_entry_get_text((GtkEntry *) m->gtkgeneral.icone)) > 0) {
		if (gpndPathIsRelative((char *) gtk_entry_get_text((GtkEntry *) m->gtkgeneral.icone))) {
			if (!gpndFileInPNDExists(m, (char *) gtk_entry_get_text((GtkEntry *) m->gtkgeneral.icone))) {
				gpndShowError(m, "The icon specified does not exist!", GPND_MESSAGE_CHILD_WINDOW);
				return;
			}
			printf("Is relative\n");
		}
	}

	if (strlen(gtk_entry_get_text((GtkEntry *) m->gtkgeneral.versione)) > 0) {
		temp = (char *) gtk_entry_get_text((GtkEntry *) m->gtkgeneral.versione);
		printf("Now: %s\n", temp);
		if ((temp = strstr(temp, ",")) != NULL) {
			temp++;
			printf("Now: %s\n", temp);
			if ((temp = strstr(temp, ",")) != NULL) {
				temp++;
				if ((temp = strstr(temp, ",")) != NULL) {
				} else {
					gpndShowError(m, "Version number must have four values separated by comma!", GPND_MESSAGE_CHILD_WINDOW);
					return;
				}
			} else {
				gpndShowError(m, "Version number must have four values separated by comma!", GPND_MESSAGE_CHILD_WINDOW);
				return;
			}
		} else {
			gpndShowError(m, "Version number must have four values separated by comma!", GPND_MESSAGE_CHILD_WINDOW);
			return;
		}
	} else {
		gpndShowError(m, "You must specify a version number!", GPND_MESSAGE_CHILD_WINDOW);
		return;
	}
					
	
	
	m->app[m->activeApp].author = realloc(m->app[m->activeApp].author, strlen(gtk_entry_get_text((GtkEntry *) m->gtkgeneral.authore)) + 1);
	if (m->app[m->activeApp].author != NULL) {
		sprintf(m->app[m->activeApp].author, "%s", gtk_entry_get_text((GtkEntry *) m->gtkgeneral.authore));
		gpndEscapeXMLData(m->app[m->activeApp].author);
	}
	
	
	m->app[m->activeApp].http = realloc(m->app[m->activeApp].http, strlen(gtk_entry_get_text((GtkEntry *) m->gtkgeneral.httpe)) + 1);
	if (m->app[m->activeApp].http != NULL) {
		sprintf(m->app[m->activeApp].http, "%s", gtk_entry_get_text((GtkEntry *) m->gtkgeneral.httpe));
		gpndEscapeXMLData(m->app[m->activeApp].http);
	}

	m->app[m->activeApp].mail = realloc(m->app[m->activeApp].mail, strlen(gtk_entry_get_text((GtkEntry *) m->gtkgeneral.maile)) + 1);
	if (m->app[m->activeApp].mail != NULL) {
		sprintf(m->app[m->activeApp].mail, "%s", gtk_entry_get_text((GtkEntry *) m->gtkgeneral.maile));
		gpndEscapeXMLData(m->app[m->activeApp].mail);
	}

	printf("Herro!\n");

	m->app[m->activeApp].icon = realloc(m->app[m->activeApp].icon, strlen(gtk_entry_get_text((GtkEntry *) m->gtkgeneral.icone)) + 1);
	if (m->app[m->activeApp].icon != NULL) {
		sprintf(m->app[m->activeApp].icon, "%s", gtk_entry_get_text((GtkEntry *) m->gtkgeneral.icone));
		gpndEscapeXMLData(m->app[m->activeApp].icon);
	}

	printf("Herro!\n");

	m->app[m->activeApp].version = realloc(m->app[m->activeApp].version, strlen(gtk_entry_get_text((GtkEntry *) m->gtkgeneral.versione)) + 1);
	if (m->app[m->activeApp].version != NULL) {
		sprintf(m->app[m->activeApp].version, "%s", gtk_entry_get_text((GtkEntry *) m->gtkgeneral.versione));
		gpndEscapeXMLData(m->app[m->activeApp].version);
	}

	gpndWindowKill(NULL, m);
	m->app[m->activeApp].flags |= 0x100;
	gpndUpdateButtonSensitivity(m);
	
	printf("Updated general data\n");

	return;
}


void gpndGetRuntimeData(GtkWidget *widget, gpointer data) {
	GPND_MAIN *m;
	m = (GPND_MAIN *) data;

	if (strlen(gtk_entry_get_text((GtkEntry *) m->gtkruntime.uide)) == 0) {
		gpndShowError(m, "You must specify a unique ID!", GPND_MESSAGE_CHILD_WINDOW);
		return;
	}

	if (strlen(gtk_entry_get_text((GtkEntry *) m->gtkruntime.execfilee)) == 0) {
		gpndShowError(m, "You must specify the application binary!", GPND_MESSAGE_CHILD_WINDOW);
		return;
	} else {
		if (!gpndFileInPNDExists(m, (char *) gtk_entry_get_text((GtkEntry *) m->gtkruntime.execfilee))) {
			gpndShowError(m, "The application binary specified does not exist!", GPND_MESSAGE_CHILD_WINDOW);
			return;
		}
	}

	if (strlen(gtk_entry_get_text((GtkEntry *) m->gtkruntime.appdatae)) > 0) {
		if (gpndArgIsPath((char *) gtk_entry_get_text((GtkEntry *) m->gtkruntime.appdatae))) {
			gpndShowError(m, "Appdata directory must be a name, not a path!", GPND_MESSAGE_CHILD_WINDOW);
			return;
		}
	}
	
	m->app[m->activeApp].uid = realloc(m->app[m->activeApp].uid, strlen(gtk_entry_get_text((GtkEntry *) m->gtkruntime.uide)));
	if (m->app[m->activeApp].uid != NULL) {
		sprintf(m->app[m->activeApp].uid, "%s", gtk_entry_get_text((GtkEntry *) m->gtkruntime.uide));
		gpndEscapeXMLData(m->app[m->activeApp].uid);
	}

	m->app[m->activeApp].appdata = realloc(m->app[m->activeApp].appdata, strlen(gtk_entry_get_text((GtkEntry *) m->gtkruntime.appdatae)));
	if (m->app[m->activeApp].appdata != NULL) {
		sprintf(m->app[m->activeApp].appdata, "%s", gtk_entry_get_text((GtkEntry *) m->gtkruntime.appdatae));
		gpndEscapeXMLData(m->app[m->activeApp].appdata);
	}

	m->app[m->activeApp].startdir = realloc(m->app[m->activeApp].startdir, strlen(gtk_entry_get_text((GtkEntry *) m->gtkruntime.startdire)));
	if (m->app[m->activeApp].startdir != NULL) {
		sprintf(m->app[m->activeApp].startdir, "%s", gtk_entry_get_text((GtkEntry *) m->gtkruntime.startdire));
		gpndEscapeXMLData(m->app[m->activeApp].startdir);
	}

	m->app[m->activeApp].execfile = realloc(m->app[m->activeApp].execfile, strlen(gtk_entry_get_text((GtkEntry *) m->gtkruntime.execfilee)));
	if (m->app[m->activeApp].execfile != NULL) {
		sprintf(m->app[m->activeApp].execfile, "%s", gtk_entry_get_text((GtkEntry *) m->gtkruntime.execfilee));
		gpndEscapeXMLData(m->app[m->activeApp].execfile);
	}

	m->app[m->activeApp].args = realloc(m->app[m->activeApp].args, strlen(gtk_entry_get_text((GtkEntry *) m->gtkruntime.argse)));
	if (m->app[m->activeApp].args != NULL) {
		sprintf(m->app[m->activeApp].args, "%s", gtk_entry_get_text((GtkEntry *) m->gtkruntime.argse));
		gpndEscapeXMLData(m->app[m->activeApp].args);
	}

	m->app[m->activeApp].cpufreq = gtk_combo_box_get_active((GtkComboBox *) m->gtkruntime.cpufreqe);
	m->app[m->activeApp].x11 = gtk_combo_box_get_active((GtkComboBox *) m->gtkruntime.x11e);
	m->app[m->activeApp].allowMultitasking = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(m->gtkruntime.allowMultitaskinge));
	m->app[m->activeApp].standalone = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(m->gtkruntime.standalonee));
	

	gpndWindowKill(NULL, m);
	m->app[m->activeApp].flags |= 0x200;
	gpndUpdateButtonSensitivity(m);
	
	printf("Updated appinfo data\n");

	return;
}


void gpndMakeCatDiag(GtkWidget *widget, gpointer data) {
	GPND_MAIN *m;
	m = (GPND_MAIN *) data;
	GtkTreeSelection *selection;
	int i;
	char *list;
	
	if (m->flags & 0x1)
		return;
	else {
		m->flags |= 0x1; 
		gpndUpdateButtonSensitivity(m);
	}
	
	m->gtkcat.window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_default_size(GTK_WINDOW(m->gtkcat.window), 400, 300);
	gtk_window_set_title(GTK_WINDOW(m->gtkcat.window), "gPND maker - Menu categories");
	gtk_container_set_border_width(GTK_CONTAINER(m->gtkcat.window), 5);
	
	m->gtkcat.table = gtk_table_new(5, 4, FALSE);
	gtk_container_add(GTK_CONTAINER(m->gtkcat.window), m->gtkcat.table);
	m->gtkcat.listscroll = gtk_scrolled_window_new(NULL, NULL);
	
	m->gtkcat.catl = gtk_label_new("Category");
	m->gtkcat.subl = gtk_label_new("Subcategory");

	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(m->gtkcat.listscroll), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW(m->gtkcat.listscroll), GTK_SHADOW_ETCHED_IN);
	
	m->gtkcat.list = gtk_tree_view_new();
	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(m->gtkcat.list), FALSE);
	gpndInitListView(m, m->gtkcat.list);


	m->gtkcat.addbtn = gtk_button_new_with_label("Add");
	gtk_widget_set_size_request(m->gtkcat.addbtn, 80, 30 );
	m->gtkcat.modbtn = gtk_button_new_with_label("Modify");
	gtk_widget_set_size_request(m->gtkcat.modbtn, 80, 30 );
	m->gtkcat.delbtn = gtk_button_new_with_label("Delete");
	gtk_widget_set_size_request(m->gtkcat.delbtn, 80, 30 );
	m->gtkcat.okbtn = gtk_button_new_with_label("OK");
	gtk_widget_set_size_request(m->gtkcat.okbtn, 80, 30 );
	
	gtk_table_attach(GTK_TABLE(m->gtkcat.table), m->gtkcat.catl, 0, 1, 0, 1, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkcat.table), m->gtkcat.subl, 0, 1, 1, 2, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);

	list = NULL;
	i = 0;
	do {
		if (m->app[m->activeApp].cat[i].flags & GPND_APP_APPISVALID) {
			list = realloc(list, strlen(m->app[m->activeApp].cat[i].subcat) + 4);
			sprintf(list, "%i/%s", m->app[m->activeApp].cat[i].cat, m->app[m->activeApp].cat[i].subcat);
			gpndAddToListView(m, m->gtkcat.list, list);
		}
		i++;
	} while (m->app[m->activeApp].cat[i - 1].flags & GPND_APP_THEREISNEXTAPP);

	free(list);
	
	m->gtkcat.sube = gtk_entry_new();
	
	m->gtkcat.cate = gtk_combo_box_new_text();
	
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(m->gtkcat.list));

	gtk_combo_box_append_text(GTK_COMBO_BOX(m->gtkcat.cate), "AudioVideo");
	gtk_combo_box_append_text(GTK_COMBO_BOX(m->gtkcat.cate), "Audio");
	gtk_combo_box_append_text(GTK_COMBO_BOX(m->gtkcat.cate), "Video");
	gtk_combo_box_append_text(GTK_COMBO_BOX(m->gtkcat.cate), "Development");
	gtk_combo_box_append_text(GTK_COMBO_BOX(m->gtkcat.cate), "Education");
	gtk_combo_box_append_text(GTK_COMBO_BOX(m->gtkcat.cate), "Game");
	gtk_combo_box_append_text(GTK_COMBO_BOX(m->gtkcat.cate), "Graphics");
	gtk_combo_box_append_text(GTK_COMBO_BOX(m->gtkcat.cate), "Network");
	gtk_combo_box_append_text(GTK_COMBO_BOX(m->gtkcat.cate), "Office");
	gtk_combo_box_append_text(GTK_COMBO_BOX(m->gtkcat.cate), "Settings");
	gtk_combo_box_append_text(GTK_COMBO_BOX(m->gtkcat.cate), "System");
	gtk_combo_box_append_text(GTK_COMBO_BOX(m->gtkcat.cate), "Utility");
	gtk_combo_box_set_active((GtkComboBox *) m->gtkcat.cate, 0);
	
	gtk_container_add(GTK_CONTAINER(m->gtkcat.listscroll), m->gtkcat.list);
	
	
	gtk_table_attach(GTK_TABLE(m->gtkcat.table), m->gtkcat.cate, 1, 5, 0, 1, 
		GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkcat.table), m->gtkcat.sube, 1, 5, 1, 2, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkcat.table), m->gtkcat.addbtn, 0, 1, 2, 3, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkcat.table), m->gtkcat.modbtn, 1, 2, 2, 3, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkcat.table), m->gtkcat.delbtn, 2, 3, 2, 3, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkcat.table), m->gtkcat.listscroll, 0, 5, 3, 4, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_EXPAND, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkcat.table), m->gtkcat.okbtn, 3, 4, 4, 5, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	


	m->currentWindow = m->gtkcat.window;
	
	g_signal_connect(G_OBJECT(m->gtkcat.addbtn), "clicked", G_CALLBACK(gpndAddCat), (gpointer) m);
	g_signal_connect(G_OBJECT(m->gtkcat.modbtn), "clicked", G_CALLBACK(gpndCatModifyEntry), (gpointer) m);
	g_signal_connect(G_OBJECT(m->gtkcat.delbtn), "clicked", G_CALLBACK(gpndCatDel), (gpointer) m);
	/*g_signal_connect(G_OBJECT(m->gtkndesc.okbtn), "clicked", G_CALLBACK(gpndGetNDesc), (gpointer) m);*/
	g_signal_connect(selection, "changed", G_CALLBACK(gpndOnCatListUpdate), (gpointer) m);
	g_signal_connect(G_OBJECT(m->gtkcat.okbtn), "clicked", G_CALLBACK(gpndWindowKill), (gpointer) m);
	m->childKillID = g_signal_connect(m->currentWindow, "destroy", G_CALLBACK(gpndWindowKill), (gpointer) m);

	gtk_widget_show_all(m->currentWindow);
	
	
	return;
}


void gpndMakeNameDescDiag(GtkWidget *widget, gpointer data) {
	GPND_MAIN *m;
	m = (GPND_MAIN *) data;
	GtkTreeSelection *selection;
	int i;
	
	if (m->flags & 0x1)
		return;
	else {
		m->flags |= 0x1; 
		gpndUpdateButtonSensitivity(m);
	}
	
	m->gtkndesc.window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_default_size(GTK_WINDOW(m->gtkndesc.window), 400, 300);
	gtk_window_set_title(GTK_WINDOW(m->gtkndesc.window), "gPND maker - Name and description");
	gtk_container_set_border_width(GTK_CONTAINER(m->gtkndesc.window), 5);

	m->gtkndesc.vbox = gtk_vbox_new(FALSE, 5);
	m->gtkndesc.btnbox = gtk_hbox_new(FALSE, 5);
	m->gtkndesc.table = gtk_table_new(6, 2, FALSE);
	gtk_container_add(GTK_CONTAINER(m->gtkndesc.window), m->gtkndesc.vbox);
	gtk_container_add(GTK_CONTAINER(m->gtkndesc.vbox), m->gtkndesc.table);
	m->gtkndesc.descscroll = gtk_scrolled_window_new(NULL, NULL);
	m->gtkndesc.listscroll = gtk_scrolled_window_new(NULL, NULL);
	
	m->gtkndesc.langl = gtk_label_new("Locale");
	m->gtkndesc.namel = gtk_label_new("Name of application");
	m->gtkndesc.descl = gtk_label_new("Description");

	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(m->gtkndesc.listscroll), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW(m->gtkndesc.listscroll), GTK_SHADOW_ETCHED_IN);
	
	m->gtkndesc.list = gtk_tree_view_new();
	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(m->gtkndesc.list), FALSE);
	gpndInitListView(m, m->gtkndesc.list);


	m->gtkndesc.addbtn = gtk_button_new_with_label("Add");
	gtk_widget_set_size_request(m->gtkndesc.addbtn, 80, 30 );
	m->gtkndesc.modbtn = gtk_button_new_with_label("Modify");
	gtk_widget_set_size_request(m->gtkndesc.modbtn, 80, 30 );
	m->gtkndesc.delbtn = gtk_button_new_with_label("Delete");
	gtk_widget_set_size_request(m->gtkndesc.delbtn, 80, 30 );
	m->gtkndesc.okbtn = gtk_button_new_with_label("OK");
	gtk_widget_set_size_request(m->gtkndesc.okbtn, 80, 30 );
	
	gtk_table_attach(GTK_TABLE(m->gtkndesc.table), m->gtkndesc.langl, 0, 1, 0, 1, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkndesc.table), m->gtkndesc.namel, 0, 1, 1, 2, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkndesc.table), m->gtkndesc.descl, 0, 1, 2, 3, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);


	i = 0;
	do {
		if (m->app[m->activeApp].ndesc[i].flags & GPND_APP_APPISVALID)
			gpndAddToListView(m, m->gtkndesc.list, m->app[m->activeApp].ndesc[i].lang);
		i++;
	} while (m->app[m->activeApp].ndesc[i - 1].flags & GPND_APP_THEREISNEXTAPP);
	
	m->gtkndesc.desce = gtk_text_view_new();
	gtk_text_view_set_wrap_mode((GtkTextView *) m->gtkndesc.desce, GTK_WRAP_WORD);
	m->gtkndesc.namee = gtk_entry_new();
	
	m->gtkndesc.lange = gtk_entry_new();
	
	
	//gtk_combo_box_set_active((GtkComboBox *) m->gtkndesc.lange, 69);
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(m->gtkndesc.list));

	gtk_container_add(GTK_CONTAINER(m->gtkndesc.descscroll), m->gtkndesc.desce);
	gtk_container_add(GTK_CONTAINER(m->gtkndesc.listscroll), m->gtkndesc.list);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(m->gtkndesc.descscroll), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW(m->gtkndesc.descscroll), GTK_SHADOW_ETCHED_IN);
	
	
	gtk_table_attach(GTK_TABLE(m->gtkndesc.table), m->gtkndesc.lange, 1, 2, 0, 1, 
		GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkndesc.table), m->gtkndesc.namee, 1, 2, 1, 2, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkndesc.table), m->gtkndesc.descscroll, 1, 2, 2, 3, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_EXPAND, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkndesc.table), m->gtkndesc.btnbox, 0, 2, 3, 4, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkndesc.table), m->gtkndesc.listscroll, 0, 2, 4, 5, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_EXPAND, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkndesc.table), m->gtkndesc.okbtn, 0, 2, 5, 6, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	


	gtk_container_add(GTK_CONTAINER(m->gtkndesc.btnbox), m->gtkndesc.addbtn);
	gtk_container_add(GTK_CONTAINER(m->gtkndesc.btnbox), m->gtkndesc.modbtn);
	gtk_container_add(GTK_CONTAINER(m->gtkndesc.btnbox), m->gtkndesc.delbtn);
	gtk_widget_set_size_request(m->gtkndesc.addbtn, 80, 30 );
	gtk_widget_set_size_request(m->gtkndesc.modbtn, 80, 30 );
	gtk_widget_set_size_request(m->gtkndesc.delbtn, 80, 30 );


	m->currentWindow = m->gtkndesc.window;
	
	g_signal_connect(G_OBJECT(m->gtkndesc.addbtn), "clicked", G_CALLBACK(gpndAddNDesc), (gpointer) m);
	g_signal_connect(G_OBJECT(m->gtkndesc.modbtn), "clicked", G_CALLBACK(gpndNDescModifyEntry), (gpointer) m);
	g_signal_connect(G_OBJECT(m->gtkndesc.delbtn), "clicked", G_CALLBACK(gpndNDescDel), (gpointer) m);
	g_signal_connect(G_OBJECT(m->gtkndesc.okbtn), "clicked", G_CALLBACK(gpndGetNDesc), (gpointer) m);
	g_signal_connect(selection, "changed", G_CALLBACK(gpndOnNDescListUpdate), (gpointer) m);
	//g_signal_connect(G_OBJECT(m->gtkruntime.canbtn), "clicked", G_CALLBACK(gpndWindowKill), (gpointer) m);
	m->childKillID = g_signal_connect(m->currentWindow, "destroy", G_CALLBACK(gpndWindowKill), (gpointer) m);

	gtk_widget_show_all(m->currentWindow);
	
	
	return;
}


void gpndMakeRuntimeDiag(GtkWidget *widget, gpointer data) {
	GPND_MAIN *m;
	m = (GPND_MAIN *) data;
	
	
	if (m->flags & 0x1)
		return;
	else {
		m->flags |= 0x1; 
		gpndUpdateButtonSensitivity(m);
	}
	
	m->gtkruntime.window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_default_size(GTK_WINDOW(m->gtkruntime.window), 200, 100);
	gtk_window_set_title(GTK_WINDOW(m->gtkruntime.window), "gPND maker - Runtime settings");
	gtk_container_set_border_width(GTK_CONTAINER(m->gtkruntime.window), 5);
	
	m->gtkruntime.table = gtk_table_new(10, 2, FALSE);
	gtk_container_add(GTK_CONTAINER(m->gtkruntime.window), m->gtkruntime.table);
	
	m->gtkruntime.uidl = gtk_label_new("Unique ID");
	m->gtkruntime.appdatal = gtk_label_new("Appdata directory");
	m->gtkruntime.startdirl = gtk_label_new("Starting directory");
	m->gtkruntime.x11l = gtk_label_new("x11 mode");
	m->gtkruntime.execfilel = gtk_label_new("Application binary");
	m->gtkruntime.argsl = gtk_label_new("Arguments");
	m->gtkruntime.cpufreql = gtk_label_new("CPU frequency");
	m->gtkruntime.allowMultitaskingl = gtk_label_new("Allow multitasking");
	m->gtkruntime.standalonel = gtk_label_new("Requires inputfile");
	
	m->gtkruntime.canbtn = gtk_button_new_with_label("Cancel");
	gtk_widget_set_size_request(m->gtkruntime.canbtn, 80, 30 );
	m->gtkruntime.okbtn = gtk_button_new_with_label("OK");
	gtk_widget_set_size_request(m->gtkruntime.okbtn, 80, 30 );
	
	gtk_table_attach(GTK_TABLE(m->gtkruntime.table), m->gtkruntime.uidl, 0, 1, 0, 1, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkruntime.table), m->gtkruntime.appdatal, 0, 1, 1, 2, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkruntime.table), m->gtkruntime.startdirl, 0, 1, 2, 3, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkruntime.table), m->gtkruntime.x11l, 0, 1, 3, 4, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkruntime.table), m->gtkruntime.execfilel, 0, 1, 4, 5, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkruntime.table), m->gtkruntime.argsl, 0, 1, 5, 6, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkruntime.table), m->gtkruntime.cpufreql, 0, 1, 6, 7, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkruntime.table), m->gtkruntime.allowMultitaskingl, 0, 1, 7, 8, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkruntime.table), m->gtkruntime.standalonel, 0, 1, 8, 9, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkruntime.table), m->gtkruntime.canbtn, 0, 1, 9, 10, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	
	
	m->gtkruntime.uide = gtk_entry_new();
	if (m->app[m->activeApp].uid != NULL)
		gtk_entry_set_text((GtkEntry *) m->gtkruntime.uide, m->app[m->activeApp].uid);
	
	m->gtkruntime.appdatae = gtk_entry_new();
	if (m->app[m->activeApp].appdata != NULL)
		gtk_entry_set_text((GtkEntry *) m->gtkruntime.appdatae, m->app[m->activeApp].appdata);
	
	m->gtkruntime.startdire = gtk_entry_new();
	if (m->app[m->activeApp].startdir != NULL)
		gtk_entry_set_text((GtkEntry *) m->gtkruntime.startdire, m->app[m->activeApp].startdir);
	
	m->gtkruntime.execfilee = gtk_entry_new();
	if (m->app[m->activeApp].execfile != NULL)
		gtk_entry_set_text((GtkEntry *) m->gtkruntime.execfilee, m->app[m->activeApp].execfile);
	
	m->gtkruntime.argse = gtk_entry_new();
	if (m->app[m->activeApp].args != NULL)
		gtk_entry_set_text((GtkEntry *) m->gtkruntime.argse, m->app[m->activeApp].args);
	
	
	m->gtkruntime.x11e = gtk_combo_box_new_text();
	gtk_combo_box_append_text(GTK_COMBO_BOX(m->gtkruntime.x11e), "Required");
	gtk_combo_box_append_text(GTK_COMBO_BOX(m->gtkruntime.x11e), "No X11");
	gtk_combo_box_append_text(GTK_COMBO_BOX(m->gtkruntime.x11e), "Don\'t care");

	gtk_combo_box_set_active((GtkComboBox *) m->gtkruntime.x11e, m->app[m->activeApp].x11);
	
	m->gtkruntime.cpufreqe = gtk_combo_box_new_text();
	gtk_combo_box_append_text(GTK_COMBO_BOX(m->gtkruntime.cpufreqe), "No change");
	gtk_combo_box_append_text(GTK_COMBO_BOX(m->gtkruntime.cpufreqe), "200");
	gtk_combo_box_append_text(GTK_COMBO_BOX(m->gtkruntime.cpufreqe), "300");
	gtk_combo_box_append_text(GTK_COMBO_BOX(m->gtkruntime.cpufreqe), "400");
	gtk_combo_box_append_text(GTK_COMBO_BOX(m->gtkruntime.cpufreqe), "500");
	gtk_combo_box_append_text(GTK_COMBO_BOX(m->gtkruntime.cpufreqe), "600");

	gtk_combo_box_set_active((GtkComboBox *) m->gtkruntime.cpufreqe, m->app[m->activeApp].cpufreq);
	
	m->gtkruntime.allowMultitaskinge = gtk_check_button_new_with_label("");
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m->gtkruntime.allowMultitaskinge), m->app[m->activeApp].allowMultitasking);
	
	m->gtkruntime.standalonee = gtk_check_button_new_with_label("");
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m->gtkruntime.standalonee), m->app[m->activeApp].standalone);
	
	gtk_table_attach(GTK_TABLE(m->gtkruntime.table), m->gtkruntime.uide, 1, 2, 0, 1, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkruntime.table), m->gtkruntime.appdatae, 1, 2, 1, 2, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkruntime.table), m->gtkruntime.startdire, 1, 2, 2, 3, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkruntime.table), m->gtkruntime.x11e, 1, 2, 3, 4, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkruntime.table), m->gtkruntime.execfilee, 1, 2, 4, 5, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkruntime.table), m->gtkruntime.argse, 1, 2, 5, 6, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkruntime.table), m->gtkruntime.cpufreqe, 1, 2, 6, 7, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkruntime.table), m->gtkruntime.allowMultitaskinge, 1, 2, 7, 8, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkruntime.table), m->gtkruntime.standalonee, 1, 2, 8, 9, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkruntime.table), m->gtkruntime.okbtn, 1, 2, 9,10, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);

	m->currentWindow = m->gtkruntime.window;
	
	g_signal_connect(G_OBJECT(m->gtkruntime.okbtn), "clicked", G_CALLBACK(gpndGetRuntimeData), (gpointer) m);
	g_signal_connect(G_OBJECT(m->gtkruntime.canbtn), "clicked", G_CALLBACK(gpndWindowKill), (gpointer) m);
	m->childKillID = g_signal_connect(m->currentWindow, "destroy", G_CALLBACK(gpndWindowKill), (gpointer) m);

	gtk_widget_show_all(m->currentWindow);
	
	
	return;
}


void gpndMakeSavePNDDiag(GtkWidget *widget, gpointer data) {
	GPND_MAIN *m;
	m = (GPND_MAIN *) data;

	if (m->flags & 0x1)
		return;
	else {
		m->flags |= 0x1; 
		gpndUpdateButtonSensitivity(m);
	}
	
	m->currentWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_default_size(GTK_WINDOW(m->currentWindow), 300, 50);
	gtk_window_set_title(GTK_WINDOW(m->currentWindow), "gPND maker - Save PND");
	gtk_container_set_border_width(GTK_CONTAINER(m->currentWindow), 5);
	
	m->gtksavepnd.table = gtk_table_new(4, 3, FALSE);
	gtk_container_add(GTK_CONTAINER(m->currentWindow), m->gtksavepnd.table);
	
	m->gtksavepnd.savetol = gtk_label_new("Filename");
	m->gtksavepnd.fsl = gtk_label_new("PND File system");

	m->gtksavepnd.canbtn = gtk_button_new_with_label("Cancel");
	gtk_widget_set_size_request(m->gtksavepnd.canbtn, 80, 30 );
	m->gtksavepnd.okbtn = gtk_button_new_with_label("OK");
	gtk_widget_set_size_request(m->gtksavepnd.okbtn, 80, 30 );
	m->gtksavepnd.browsebtn = gtk_button_new_with_label("Browse");
	gtk_widget_set_size_request(m->gtksavepnd.okbtn, 80, 30 );

	m->gtksavepnd.fse = gtk_combo_box_new_text();

	if (m->flags & GPND_MAIN_GENISOIMAGE)
		gtk_combo_box_append_text(GTK_COMBO_BOX(m->gtksavepnd.fse), "ISO9660");
	if (m->flags & GPND_MAIN_GENSQUASHFS)
		gtk_combo_box_append_text(GTK_COMBO_BOX(m->gtksavepnd.fse), "SquashFS");
	if (m->flags & GPND_MAIN_GENCRAMFS)
		gtk_combo_box_append_text(GTK_COMBO_BOX(m->gtksavepnd.fse), "CramFS");
	
	gtk_combo_box_set_active((GtkComboBox *) m->gtksavepnd.fse, 0);

	gtk_table_attach(GTK_TABLE(m->gtksavepnd.table), m->gtksavepnd.fsl, 0, 1, 0, 1, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtksavepnd.table), m->gtksavepnd.savetol, 0, 1, 1, 2, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtksavepnd.table), m->gtksavepnd.canbtn, 1, 2, 3, 4, 
		GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_SHRINK, 5, 5);

	m->gtksavepnd.savetoe = gtk_entry_new();
	
	gtk_table_attach(GTK_TABLE(m->gtksavepnd.table), m->gtksavepnd.fse, 1, 3, 0, 1, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtksavepnd.table), m->gtksavepnd.savetoe, 0, 2, 2, 3, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtksavepnd.table), m->gtksavepnd.browsebtn, 2, 3, 2, 3, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtksavepnd.table), m->gtksavepnd.okbtn, 2, 3, 3, 4, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	

	gtk_widget_show_all(m->currentWindow);

	g_signal_connect(G_OBJECT(m->gtksavepnd.okbtn), "clicked", G_CALLBACK(gpndMakePND), (gpointer) m);
	g_signal_connect(G_OBJECT(m->gtksavepnd.browsebtn), "clicked", G_CALLBACK(gpndGetTargetPND), (gpointer) m);
	g_signal_connect(G_OBJECT(m->gtksavepnd.canbtn), "clicked", G_CALLBACK(gpndWindowKill), (gpointer) m);
	m->childKillID = g_signal_connect(m->currentWindow, "destroy", G_CALLBACK(gpndWindowKill), (gpointer) m);

	return;
}


void gpndMakeAppGeneralDiag(GtkWidget *widget, gpointer data) {
	GPND_MAIN *m;
	m = (GPND_MAIN *) data;

	if (m->flags & 0x1)
		return;
	else {
		m->flags |= 0x1; 
		gpndUpdateButtonSensitivity(m);
	}
	
	m->currentWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_default_size(GTK_WINDOW(m->currentWindow), 200, 100);
	gtk_window_set_title(GTK_WINDOW(m->currentWindow), "gPND maker - General information");
	gtk_container_set_border_width(GTK_CONTAINER(m->currentWindow), 5);
	
	m->gtkgeneral.table = gtk_table_new(7, 2, FALSE);
	gtk_container_add(GTK_CONTAINER(m->currentWindow), m->gtkgeneral.table);
	
	m->gtkgeneral.authorl = gtk_label_new("Author");
	m->gtkgeneral.httpl = gtk_label_new("Homepage");
	m->gtkgeneral.maill = gtk_label_new("E-mail");
	//m->gtkgeneral.descl = gtk_label_new("Description");
	m->gtkgeneral.iconl = gtk_label_new("Menu icon file");
	m->gtkgeneral.versionl = gtk_label_new("Version");

	m->gtkgeneral.canbtn = gtk_button_new_with_label("Cancel");
	gtk_widget_set_size_request(m->gtkgeneral.canbtn, 80, 30 );
	m->gtkgeneral.okbtn = gtk_button_new_with_label("OK");
	gtk_widget_set_size_request(m->gtkgeneral.okbtn, 80, 30 );

	gtk_table_attach(GTK_TABLE(m->gtkgeneral.table), m->gtkgeneral.authorl, 0, 1, 0, 1, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkgeneral.table), m->gtkgeneral.httpl, 0, 1, 1, 2, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkgeneral.table), m->gtkgeneral.maill, 0, 1, 2, 3, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkgeneral.table), m->gtkgeneral.versionl, 0, 1, 3, 4, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkgeneral.table), m->gtkgeneral.iconl, 0, 1, 4, 5, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	/*gtk_table_attach(GTK_TABLE(m->gtkgeneral.table), m->gtkgeneral.descl, 0, 1, 5, 6, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);*/
	gtk_table_attach(GTK_TABLE(m->gtkgeneral.table), m->gtkgeneral.canbtn, 0, 1, 6, 7, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);

	m->gtkgeneral.authore = gtk_entry_new();
	
	if (m->app[m->activeApp].author != NULL)
		gtk_entry_set_text((GtkEntry *) m->gtkgeneral.authore, m->app[m->activeApp].author);
	m->gtkgeneral.httpe = gtk_entry_new();
	
	if (m->app[m->activeApp].http != NULL)
		gtk_entry_set_text((GtkEntry *) m->gtkgeneral.httpe, m->app[m->activeApp].http);
	m->gtkgeneral.maile = gtk_entry_new();
	
	if (m->app[m->activeApp].mail != NULL)
		gtk_entry_set_text((GtkEntry *) m->gtkgeneral.maile, m->app[m->activeApp].mail);
	m->gtkgeneral.versione = gtk_entry_new();
	
	if (m->app[m->activeApp].version != NULL)
		gtk_entry_set_text((GtkEntry *) m->gtkgeneral.versione, m->app[m->activeApp].version);
	else
		gtk_entry_set_text((GtkEntry *) m->gtkgeneral.versione, "1,0,0,0");
	
	m->gtkgeneral.icone = gtk_entry_new();
	
	if (m->app[m->activeApp].icon != NULL)
		gtk_entry_set_text((GtkEntry *) m->gtkgeneral.icone, m->app[m->activeApp].icon);
	//m->gtkgeneral.desce = gtk_entry_new();
	
	/*if (m->app[m->activeApp].desc != NULL)
		gtk_entry_set_text((GtkEntry *) m->gtkgeneral.desce, m->app[m->activeApp].desc);*/
	
	gtk_table_attach(GTK_TABLE(m->gtkgeneral.table), m->gtkgeneral.authore, 1, 2, 0, 1, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkgeneral.table), m->gtkgeneral.httpe, 1, 2, 1, 2, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkgeneral.table), m->gtkgeneral.maile, 1, 2, 2, 3, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkgeneral.table), m->gtkgeneral.versione, 1, 2, 3, 4, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	gtk_table_attach(GTK_TABLE(m->gtkgeneral.table), m->gtkgeneral.icone, 1, 2, 4, 5, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	/*gtk_table_attach(GTK_TABLE(m->gtkgeneral.table), m->gtkgeneral.desce, 1, 2, 5, 6, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);*/
	gtk_table_attach(GTK_TABLE(m->gtkgeneral.table), m->gtkgeneral.okbtn, 1, 2, 6, 7, 
		GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 5, 5);
	

	gtk_widget_show_all(m->currentWindow);

	g_signal_connect(G_OBJECT(m->gtkgeneral.okbtn), "clicked", G_CALLBACK(gpndGetAppGeneralData), (gpointer) m);
	g_signal_connect(G_OBJECT(m->gtkgeneral.canbtn), "clicked", G_CALLBACK(gpndWindowKill), (gpointer) m);
	m->childKillID = g_signal_connect(m->currentWindow, "destroy", G_CALLBACK(gpndWindowKill), (gpointer) m);

	return;
}


int gpndSetUpGTK(GPND_MAIN *m) {
	GtkTreeSelection *selection;
	
	m->gtkstruct.window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_default_size(GTK_WINDOW(m->gtkstruct.window), 200, 100);
	gtk_window_set_title(GTK_WINDOW(m->gtkstruct.window), "gPND maker");
	gtk_container_set_border_width(GTK_CONTAINER(m->gtkstruct.window), 5);
	
	m->gtkstruct.vbox = gtk_vbox_new(FALSE, 5);
	m->gtkstruct.hbox = gtk_hbox_new(FALSE, 5);
	m->gtkstruct.addbox = gtk_hbox_new(FALSE, 5);
	m->gtkstruct.toolbox = gtk_hbox_new(FALSE, 5);
	m->gtkstruct.mainbox = gtk_vbox_new(FALSE, 5);

	m->gtkstruct.appscroll = gtk_scrolled_window_new(NULL, NULL);

	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(m->gtkstruct.appscroll), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW(m->gtkstruct.appscroll), GTK_SHADOW_ETCHED_IN);
	
	m->gtkstruct.applist = gtk_tree_view_new();
	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(m->gtkstruct.applist), FALSE);


	gtk_container_add(GTK_CONTAINER(m->gtkstruct.window), m->gtkstruct.mainbox);
	gtk_container_add(GTK_CONTAINER(m->gtkstruct.mainbox), m->gtkstruct.addbox);
	gtk_container_add(GTK_CONTAINER(m->gtkstruct.mainbox), m->gtkstruct.toolbox);
	gtk_container_add(GTK_CONTAINER(m->gtkstruct.mainbox), m->gtkstruct.hbox);
	gtk_box_pack_start(GTK_BOX(m->gtkstruct.hbox), m->gtkstruct.appscroll, TRUE, TRUE, 5);
	gtk_container_add(GTK_CONTAINER(m->gtkstruct.appscroll), m->gtkstruct.applist);
	gtk_container_add(GTK_CONTAINER(m->gtkstruct.hbox), m->gtkstruct.vbox);
	gpndInitListView(m, m->gtkstruct.applist);
	
	//m->gtkstruct.new = gtk_button_new_with_label("New PND");
	m->gtkstruct.seldir = gtk_button_new_with_label("Select directory");
	m->gtkstruct.appinfo = gtk_button_new_with_label("General information");
	m->gtkstruct.apprun = gtk_button_new_with_label("Runtime data");
	m->gtkstruct.appndesc = gtk_button_new_with_label("Name and description");
	m->gtkstruct.cat = gtk_button_new_with_label("Menu categories");
	//m->gtkstruct.selprevpics = gtk_button_new_with_label("Set preview pics");
	m->gtkstruct.savepnd = gtk_button_new_with_label("Export to PND");

	m->gtkstruct.addApp = gtk_button_new_with_label("Add app");
	m->gtkstruct.renApp = gtk_button_new_with_label("Rename app");
	m->gtkstruct.delApp = gtk_button_new_with_label("Remove app");
	m->gtkstruct.appName = gtk_entry_new();

	gtk_box_pack_start(GTK_BOX(m->gtkstruct.toolbox), m->gtkstruct.appName, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(m->gtkstruct.toolbox), m->gtkstruct.addApp, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(m->gtkstruct.toolbox), m->gtkstruct.renApp, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(m->gtkstruct.toolbox), m->gtkstruct.delApp, TRUE, TRUE, 0);

	//gtk_box_pack_start(GTK_BOX(m->gtkstruct.vbox), m->gtkstruct.new, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(m->gtkstruct.addbox), m->gtkstruct.seldir, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(m->gtkstruct.vbox), m->gtkstruct.appinfo, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(m->gtkstruct.vbox), m->gtkstruct.apprun, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(m->gtkstruct.vbox), m->gtkstruct.appndesc, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(m->gtkstruct.vbox), m->gtkstruct.cat, TRUE, TRUE, 0);
	//gtk_box_pack_start(GTK_BOX(m->gtkstruct.vbox), m->gtkstruct.selprevpics, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(m->gtkstruct.mainbox), m->gtkstruct.savepnd, TRUE, TRUE, 0);

	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(m->gtkstruct.applist));
	
	g_signal_connect(G_OBJECT(m->gtkstruct.seldir), "clicked", G_CALLBACK(gpndGetDir), (gpointer) m);
	g_signal_connect(G_OBJECT(m->gtkstruct.appinfo), "clicked", G_CALLBACK(gpndMakeAppGeneralDiag), (gpointer) m);
	g_signal_connect(G_OBJECT(m->gtkstruct.apprun), "clicked", G_CALLBACK(gpndMakeRuntimeDiag), (gpointer) m);
	g_signal_connect(G_OBJECT(m->gtkstruct.appndesc), "clicked", G_CALLBACK(gpndMakeNameDescDiag), (gpointer) m);
	g_signal_connect(G_OBJECT(m->gtkstruct.cat), "clicked", G_CALLBACK(gpndMakeCatDiag), (gpointer) m);
	g_signal_connect(G_OBJECT(m->gtkstruct.savepnd), "clicked", G_CALLBACK(gpndMakeSavePNDDiag), (gpointer) m);
	g_signal_connect(G_OBJECT(m->gtkstruct.addApp), "clicked", G_CALLBACK(gpndAddApp), (gpointer) m);
	g_signal_connect(G_OBJECT(m->gtkstruct.renApp), "clicked", G_CALLBACK(gpndRenApp), (gpointer) m);
	g_signal_connect(G_OBJECT(m->gtkstruct.delApp), "clicked", G_CALLBACK(gpndDelApp), (gpointer) m);
	g_signal_connect(selection, "changed", G_CALLBACK(gpndOnAppListUpdate), (gpointer) m);
	g_signal_connect(m->gtkstruct.window, "destroy", G_CALLBACK(gtk_main_quit), NULL);
	
	
	gpndUpdateButtonSensitivity(m);
	
	gtk_widget_show_all(m->gtkstruct.window);
	
	return 0;
}


void gpndInit(GPND_MAIN *m) {
	if (system("genisoimage --version") == 0)
		m->flags |= GPND_MAIN_GENISOIMAGE;
	if (system("mksquashfs -version") == 0)
		m->flags |= GPND_MAIN_GENSQUASHFS;
	if (system("mkcramfs -h") == 0)
		m->flags |= GPND_MAIN_GENCRAMFS;
	
	gpndLoadLocales(m);
	gpndInitApp(m);

	if (!(m->flags & GPND_MAIN_IMAGEFS)) {
		gpndShowError(m, "You don't have any supported tools installed to generate a image file.\n\nInstall one or more of the following:\ngenisoimage, mksquashfs, mkcramfs", 0);
		exit(255);
	}

	return;
}


int main(int argc, char **argv) {
	GPND_MAIN m;
	
	gtk_init(&argc, &argv);
	gpndInitStructs(&m);
	
	gpndSetUpGTK(&m);
	gpndInit(&m);
	
	gtk_main();
	
	return 0;
}
